package dk.deranged.android.refleq;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Random;

import android.util.Log;
import android.view.View;
import android.widget.TextView;

public class RefleqMaster implements Runnable {
	private static final int POINTS_VIEW_DIGITS = 8;
	
	private List<RefleqButton> buttonsList;
	private Map<View,RefleqButton> buttonsMap;
	private PriorityQueue<RefleqButton> buttonsQueue;
	
	private int points = 0, lives = 6, step = 0;
	
	private Random rndGen = new Random();
	private PlayActivity parent;

	public RefleqMaster(PlayActivity parent, TextView pointsView, RefleqButton... rButtons) {
		buttonsList = new ArrayList<RefleqButton>();
		buttonsMap = new HashMap<View,RefleqButton>();
		buttonsQueue = new PriorityQueue<RefleqButton>();
		
		this.parent = parent;
		
		for(RefleqButton button : rButtons) {
			buttonsList.add(button);
			buttonsMap.put(button.view(),button);
			
			button.setOnClickListener(new View.OnClickListener() {
				private RefleqMaster master = RefleqMaster.this;

				public void onClick(View v) {
					master.pressed(v);
				}
			});
		}
		
		setPoints(points);
		setLife(lives);
	}
	
	public void add(RefleqButton button) {
		buttonsList.add(button);
		buttonsMap.put(button.view(), button);
	}
	
	public void remove(RefleqButton button) {
		buttonsList.remove(button);
		buttonsMap.remove(button.view());
	}
	
	public double valueFromStep(int step) {
		return 10 / step + 0.4;
	}
	
	public double pointsAtStep(int step) {
		return 4 / valueFromStep(step) + 1;
	}
	
	public void pressed(View v) {
		pressed(buttonsMap.get(v));
	}
	
	public synchronized void pressed(RefleqButton button) {
		//points
		if(button.isActive() || buttonsQueue.contains(button)) {
			dPoints((int) pointsAtStep(step));
			button.deactivate();
			if(buttonsQueue.contains(button)) buttonsQueue.remove(button);
		}
		else {
			loseLife();
		}
	}
	
	private void loseLife() {
		setLife(lives--);
	}
	
	private void setLife(int life) {
		String lifeString = "Lives: ";
		for(int i = 0; i < life; i++) lifeString += i+" ";
		
		parent.runOnUiThread(new TextViewSetter((TextView) parent.findViewById(R.id.playLivesText), lifeString));
	}
	
	private void dPoints(int value) {
		points += value;
		if(points < 0) points = 0;
		setPoints(points);
	}
	
	private void setPoints(int value) {
		String pointsString = ""+value;
		
		while(pointsString.length() < POINTS_VIEW_DIGITS) {
			pointsString = "0"+pointsString;
		}
		
		parent.runOnUiThread(new TextViewSetter((TextView) parent.findViewById(R.id.playPointsText), pointsString));
	}
	
	private void endGame() {
		parent.endGame();
	}
	
	private class TextViewSetter implements Runnable {
		private TextView view;
		private String text;
		
		public TextViewSetter(TextView view, String text) {
			this.view = view;
			this.text = text;
		}
		
		public void run() {
			view.setText(text);
		}
	}
	
	public void run() {
		step = 1;
		
		while(lives > 0) {
			long buttonLiveTime = (long) (2000 * (rndGen.nextDouble() * 0.5 + 0.75) * valueFromStep(step));
			Log.w("RefleQ","Leaking: buttonLiveTime: "+buttonLiveTime);
			
			if(rndGen.nextInt(10) > 2) { //70% chance to activate a button
				Log.w("RefleQ","Activating a button.");
				RefleqButton button;
				button = buttonsList.get(rndGen.nextInt(buttonsList.size()));
				while(button.isActive()) button = buttonsList.get(rndGen.nextInt(buttonsList.size()));
				
				button.activate(buttonLiveTime);
				buttonsQueue.add(button);
			}
			
			long sleepTime;
			if(!buttonsQueue.isEmpty()) {
				sleepTime = ((buttonsQueue.peek().deactivateTimestamp() - (System.nanoTime() / 1000)) / 2);
			}
			else {
				sleepTime = buttonLiveTime;
			}
			
			Log.w("RefleQ","Sleeping on game thread for "+sleepTime+" millis.");
			if(sleepTime > 0) {
				try {
					Thread.sleep(sleepTime);
				}
				catch(InterruptedException e) { Log.w("RefleQ","Game thread sleep interrupted!"); }
			}
			Log.w("RefleQ","Game thread resumed.");
			
			//get button
			if(!buttonsQueue.isEmpty() && buttonsQueue.peek().deactivateTimestamp() <= (System.nanoTime() / 1000)) {
				RefleqButton button = buttonsQueue.poll();
				button.deactivate();
				loseLife();
			}
			
			step++;
		}
		Log.w("RefleQ","Game has ended.");
		endGame();
	}
}
